// ********************************************************
// * 1HE VFD Clock Firmware    (c) 2009 by maltepggel.de *
// ********************************************************
// * Header File VFD Multiplex and Decoding Handler       *
// ********************************************************

#ifndef VFD_H
  #define VFD_H

  #include <avr/io.h>
  #include <util/delay.h>
  #include <avr/interrupt.h>
  
  #include "libs/portdef.h"
  
  // Globals for segment data
  volatile uint8_t segdata[6];		// Array for segment data
  volatile uint8_t segcounter;		// Counter for grid position
    
  // --- Initialize vfd ports and interrupt ---
  inline void InitVFD( void ) 
  {
   // First we set up the ports for the MAX6921 display shift register
   DISP_DDR_CLK  |= (1<<DISP_CLK);   	// Clock
   DISP_DDR_LOAD |= (1<<DISP_LOAD);  	// Load
   DISP_DDR_DATA |= (1<<DISP_DATA); 	// Data
   
   DISP_PRT_CLK  &= ~(1<<DISP_CLK);  	// Set ports to zero
   DISP_PRT_LOAD &= ~(1<<DISP_LOAD);
   DISP_PRT_DATA &= ~(1<<DISP_DATA);
  
   // And of course the port for the LM9022 heater / hv chip
   
   TCCR0B = (1<<CS01) | (1<<CS00);		// Setup prescaler (8MHz / 32)
   TIMSK |= (1<<TOIE0);				// Setup interrupt
   
   segdata[0]=0;
   segdata[1]=0;
   segdata[2]=0;
   segdata[3]=0;
   segdata[4]=0;
   segdata[5]=0;

  }
  
  // --- Send grid data to shift register ---
  inline void GridShiftOut( uint8_t data )
  {
   uint8_t i;

   for( i=5; i>0; i-- ) 
   {
    DISP_PRT_DATA &= ~(1<<DISP_DATA);    
    if(data==i) {
      DISP_PRT_DATA |= (1<<DISP_DATA);   
    }
    DISP_PRT_CLK |= (1<<DISP_CLK);
	DISP_PRT_CLK &= ~(1<<DISP_CLK);
   }
  }

  inline void UpperDotShiftOut( uint8_t data, uint8_t segcounter )
  {
   DISP_PRT_DATA &= ~(1<<DISP_DATA);
   if(data & (1<<segcounter)) {
     DISP_PRT_DATA |= (1<<DISP_DATA);   
   }
   DISP_PRT_CLK |= (1<<DISP_CLK);
   DISP_PRT_CLK &= ~(1<<DISP_CLK);
  }
  
  // --- Send segment data to shift register ---
  inline void SegShiftOut( uint8_t data )
  {
   uint8_t i;
   
   for( i=8; i>0; i-- ) 
   {
    DISP_PRT_DATA &= ~(1<<DISP_DATA);
    if ( data & 0x80 )
    {
     DISP_PRT_DATA |= (1<<DISP_DATA);
    }
    data <<= 1;
    DISP_PRT_CLK |= (1<<DISP_CLK);
	DISP_PRT_CLK &= ~(1<<DISP_CLK);
   }  
  }
  
  void DontCare( void );
  void DisplayConvert( uint8_t val, uint8_t *digit_ten, uint8_t *digit_one );
  void DisplayConvertDot( uint8_t val, uint8_t *digit_ten, uint8_t *digit_one );
  void WeekdayConvert( uint8_t val, uint8_t *digit_one, uint8_t *digit_two );
#endif
